From: Mircea Cîrjaliu <[email protected]>

This change is needed because the introspection tool can write-protect
guest page tables, exec-protect heap/stack pages and let KVM emulate
the instruction that caused these violations.

Signed-off-by: Mircea Cîrjaliu <[email protected]>
Signed-off-by: Adalbert Lazăr <[email protected]>
---
 arch/x86/kvm/mmu/mmu.c | 9 +++++++--
 1 file changed, 7 insertions(+), 2 deletions(-)

diff --git a/arch/x86/kvm/mmu/mmu.c b/arch/x86/kvm/mmu/mmu.c
index 5f0ddc6b67de..d28b7425c4f1 100644
--- a/arch/x86/kvm/mmu/mmu.c
+++ b/arch/x86/kvm/mmu/mmu.c
@@ -5636,8 +5636,13 @@ int kvm_mmu_page_fault(struct kvm_vcpu *vcpu, gva_t cr2, 
u64 error_code,
        int r, emulation_type = 0;
        bool direct = vcpu->arch.mmu->direct_map;
 
-       /* With shadow page tables, fault_address contains a GVA or nGPA.  */
-       if (vcpu->arch.mmu->direct_map) {
+       /*
+        * With shadow page tables, fault_address contains a GVA or nGPA.
+        * On a fetch fault, fault_address contains the instruction pointer.
+        */
+       if (vcpu->arch.mmu->direct_map &&
+           likely(!(error_code & PFERR_FETCH_MASK)) &&
+           (error_code & PFERR_GUEST_FINAL_MASK)) {
                vcpu->arch.gpa_available = true;
                vcpu->arch.gpa_val = cr2;
        }
_______________________________________________
Virtualization mailing list
[email protected]
https://lists.linuxfoundation.org/mailman/listinfo/virtualization

Reply via email to